import { join, relative } from 'path';
import { fileURLToPath } from 'url';
import { ESLint } from 'eslint';
import isInstalledGlobally from 'is-installed-globally';
import { format } from './formatter/index.js';
import { eslintInteractivePlugin, OVERLAPPED_PROBLEM_MESSAGE, } from './plugin/index.js';
import { unique } from './util/array.js';
import { getCacheDir } from './util/cache.js';
import { filterResultsByRuleId, scanUsedPluginsFromResults } from './util/eslint.js';
const MAX_AUTOFIX_PASSES = 10;
/**
 * Generate results to undo.
 * @param resultsOfLint The results of lint.
 * @returns The results to undo.
 */
function generateResultsToUndo(resultsOfLint) {
    return resultsOfLint.map((resultOfLint) => {
        // NOTE: THIS IS HACK.
        return { ...resultOfLint, output: resultOfLint.source };
    });
}
function hasOverlappedProblems(results) {
    return results.flatMap((result) => result.messages).some((message) => message.message === OVERLAPPED_PROBLEM_MESSAGE);
}
/**
 * Get all the rules loaded from eslintrc.
 * @param targetFilePaths The target file paths.
 * @param options The eslint option.
 * @returns The rule ids loaded from eslintrc.
 */
async function getUsedRuleIds(targetFilePaths, options) {
    const eslintToGetRules = new ESLint(options);
    const configs = await Promise.all(targetFilePaths.map(async (filePath) => eslintToGetRules.calculateConfigForFile(filePath)));
    return unique(configs.map((config) => config.rules).flatMap((rules) => Object.keys(rules)));
}
/** Default config of `Core` */
export const DEFAULT_BASE_CONFIG = {
    useEslintrc: true,
    cwd: undefined,
    cache: true,
    cacheLocation: relative(process.cwd(), join(getCacheDir(), '.eslintcache')),
    extensions: undefined,
    formatterName: 'codeframe',
    quiet: false,
    rulePaths: undefined,
    resolvePluginsRelativeTo: undefined,
};
/**
 * The core of eslint-interactive.
 * It uses ESLint's Node.js API to output a summary of problems, fix problems, apply suggestions, etc.
 */
export class Core {
    constructor(config) {
        this.config = {
            ...DEFAULT_BASE_CONFIG,
            ...config,
        };
        const { patterns, formatterName, quiet, ...baseOptions } = this.config;
        this.baseESLintOptions = baseOptions;
    }
    /**
     * Lint project.
     * @returns The results of linting
     */
    async lint() {
        const eslint = new ESLint(this.baseESLintOptions);
        let results = await eslint.lintFiles(this.config.patterns);
        if (this.config.quiet)
            results = ESLint.getErrorResults(results);
        return results;
    }
    /**
     * Returns summary of lint results.
     * @param results The lint results of the project to print summary
     */
    formatResultSummary(results) {
        var _a, _b, _c;
        // get used plugins from `results`
        const plugins = scanUsedPluginsFromResults(results);
        // get `rulesMeta` from `results`
        const eslint = new ESLint({
            ...this.baseESLintOptions,
            overrideConfig: { ...this.baseESLintOptions.overrideConfig, plugins },
        });
        // NOTE: `getRulesMetaForResults` is a feature added in ESLint 7.29.0.
        // Therefore, the function may not exist in versions lower than 7.29.0.
        const rulesMeta = (_b = (_a = eslint.getRulesMetaForResults) === null || _a === void 0 ? void 0 : _a.call(eslint, results)) !== null && _b !== void 0 ? _b : {};
        return format(results, { rulesMeta: rulesMeta, cwd: (_c = this.config.cwd) !== null && _c !== void 0 ? _c : process.cwd() });
    }
    /**
     * Returns details of lint results.
     * @param results The lint results of the project to print summary
     * @param ruleIds The rule ids to print details
     */
    async formatResultDetails(results, ruleIds) {
        var _a;
        const eslint = new ESLint(this.baseESLintOptions);
        const formatterName = (_a = this.config.formatterName) !== null && _a !== void 0 ? _a : DEFAULT_BASE_CONFIG.formatterName;
        // When eslint-interactive is installed globally, eslint-formatter-codeframe will also be installed globally.
        // On the other hand, `eslint.loadFormatter` cannot load the globally installed formatter by name. So here it loads them by path.
        const resolvedFormatterNameOrPath = isInstalledGlobally && formatterName === 'codeframe'
            ? fileURLToPath(
            // @ts-expect-error
            await import.meta.resolve('eslint-formatter-codeframe', 
            // @ts-expect-error
            await import.meta.resolve('eslint-interactive')))
            : formatterName;
        const formatter = await eslint.loadFormatter(resolvedFormatterNameOrPath);
        return formatter.format(filterResultsByRuleId(results, ruleIds));
    }
    /**
     * Run `eslint --fix`.
     * @param ruleIds The rule ids to fix
     */
    async applyAutoFixes(results, ruleIds) {
        return await this.fix(results, ruleIds, { name: 'applyAutoFixes', args: {} });
    }
    /**
     * Add disable comments per line.
     * @param results The lint results of the project to add disable comments
     * @param ruleIds The rule ids to add disable comments
     * @param description The description of the disable comments
     * @param descriptionPosition The position of the description
     */
    async disablePerLine(results, ruleIds, description, descriptionPosition) {
        return await this.fix(results, ruleIds, { name: 'disablePerLine', args: { description, descriptionPosition } });
    }
    /**
     * Add disable comments per file.
     * @param results The lint results of the project to add disable comments
     * @param ruleIds The rule ids to add disable comments
     * @param description The description of the disable comments
     * @param descriptionPosition The position of the description
     */
    async disablePerFile(results, ruleIds, description, descriptionPosition) {
        return await this.fix(results, ruleIds, { name: 'disablePerFile', args: { description, descriptionPosition } });
    }
    /**
     * Convert error to warning per file.
     * @param results The lint results of the project to convert
     * @param ruleIds The rule ids to convert
     * @param description The comment explaining the reason for converting
     */
    async convertErrorToWarningPerFile(results, ruleIds, description) {
        return await this.fix(results, ruleIds, { name: 'convertErrorToWarningPerFile', args: { description } });
    }
    /**
     * Apply suggestions.
     * @param results The lint results of the project to apply suggestions
     * @param ruleIds The rule ids to apply suggestions
     * @param filter The script to filter suggestions
     * */
    async applySuggestions(results, ruleIds, filter) {
        return await this.fix(results, ruleIds, { name: 'applySuggestions', args: { filter } });
    }
    /**
     * Make forcibly fixable and run `eslint --fix`.
     * @param results The lint results of the project to apply suggestions
     * @param ruleIds The rule ids to apply suggestions
     * @param fixableMaker The function to make `Linter.LintMessage` forcibly fixable.
     * */
    async makeFixableAndFix(results, ruleIds, fixableMaker) {
        return await this.fix(results, ruleIds, { name: 'makeFixableAndFix', args: { fixableMaker } });
    }
    /**
     * Fix source codes.
     * @param fix The fix information to do.
     */
    async fix(resultsOfLint, ruleIds, fix) {
        // NOTE: Extract only necessary results and files for performance
        const filteredResultsOfLint = filterResultsByRuleId(resultsOfLint, ruleIds);
        const targetFilePaths = filteredResultsOfLint.map((result) => result.filePath);
        const usedRuleIds = await getUsedRuleIds(targetFilePaths, this.baseESLintOptions);
        // TODO: refactor
        let results = filteredResultsOfLint;
        for (let i = 0; i < MAX_AUTOFIX_PASSES; i++) {
            const eslint = new ESLint({
                ...this.baseESLintOptions,
                // This is super hack to load ESM plugin/rule.
                // ref: https://github.com/eslint/eslint/issues/15453#issuecomment-1001200953
                plugins: {
                    'eslint-interactive': eslintInteractivePlugin,
                },
                overrideConfig: {
                    plugins: ['eslint-interactive'],
                    rules: {
                        'eslint-interactive/fix': [2, { results, ruleIds, fix }],
                        // Turn off all rules except `eslint-interactive/fix` when fixing for performance.
                        ...Object.fromEntries(usedRuleIds.map((ruleId) => [ruleId, 'off'])),
                    },
                },
                // NOTE: Only fix the `fix` rule problems.
                fix: (message) => message.ruleId === 'eslint-interactive/fix',
                // Don't interpret lintFiles arguments as glob patterns for performance.
                globInputPaths: false,
            });
            const resultsToFix = await eslint.lintFiles(targetFilePaths);
            await ESLint.outputFixes(resultsToFix);
            if (!hasOverlappedProblems(resultsToFix))
                break;
            results = await this.lint();
        }
        return async () => {
            const resultsToUndo = generateResultsToUndo(filteredResultsOfLint);
            await ESLint.outputFixes(resultsToUndo);
        };
    }
}
//# sourceMappingURL=core.js.map